package org.gcube.informationsystem.utils.documentation;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.ServiceLoader;

import org.gcube.informationsystem.base.reference.AccessType;
import org.gcube.informationsystem.base.reference.Element;
import org.gcube.informationsystem.types.TypeMapper;
import org.gcube.informationsystem.types.reference.Type;
import org.gcube.informationsystem.utils.discovery.ElementSpecilizationDiscovery;
import org.gcube.informationsystem.utils.discovery.RegistrationProvider;

/**
 * @author Luca Frosini (ISTI - CNR)
 */
public class ClassesDiscoveryGenerator {

	protected TypeListGenerator typeListGenerator;
	
	public ClassesDiscoveryGenerator() {
		
	}
	
	protected List<Package> getPackages(){
		
		List<Package> packages = new ArrayList<Package>();
			
		AccessType[] accessTypes = new AccessType[] {
			AccessType.PROPERTY, 
			AccessType.ENTITY,
			AccessType.RELATION
		};
		
		for(AccessType accessType : accessTypes) {
			Class<Element> clz = accessType.getTypeClass();
			packages.add(clz.getPackage());
		}
			
		ServiceLoader<? extends RegistrationProvider> regsitrationProviders = ServiceLoader
				.load(RegistrationProvider.class);
		for(RegistrationProvider registrationProvider : regsitrationProviders) {
			packages.addAll(registrationProvider.getPackagesToRegister());
		}
			
		return packages;
	}
	
	public void generate() throws Exception {
		List<Package> pkgs = getPackages();
		generate(pkgs.toArray(new Package[pkgs.size()]));
	}
	
	protected <E extends Element> void  discover(AccessType at, Package[] packages) throws Exception {
		ElementSpecilizationDiscovery<E> discovery = new ElementSpecilizationDiscovery<>(at.getTypeClass());
		if(Objects.nonNull(packages)) {
			Arrays.stream(packages).forEach(p -> discovery.addPackage(p));
		}
		discovery.discover();
		
		for(Class<? extends E> clz : discovery.getDiscovered()) {
			Type type = TypeMapper.createTypeDefinition(clz);
			typeListGenerator.addType(type);
		}
	}
	
	protected void generate(Package[] packages) throws Exception {
		typeListGenerator = new TypeListGenerator();
		AccessType[] types = new AccessType[] {
				AccessType.PROPERTY, 
				AccessType.RESOURCE, AccessType.FACET, 
				AccessType.CONSISTS_OF, AccessType.IS_RELATED_TO};
		for(AccessType at : types) {
			discover(at, packages);
		}
		
		typeListGenerator.generate();
		
	}
}
